/*
 * citygml4j - The Open Source Java API for CityGML
 * https://github.com/citygml4j
 * 
 * Copyright (C) 2013 - 2015,
 * Claus Nagel <claus.nagel@gmail.com>
 *
 * The citygml4j library is free software:
 * you can redistribute it and/or modify it under the terms of the
 * GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 */


import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.transform.Templates;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.stream.StreamSource;

import org.citygml4j.CityGMLContext;
import org.citygml4j.builder.CityGMLBuilder;
import org.citygml4j.model.citygml.CityGML;
import org.citygml4j.model.citygml.core.AbstractCityObject;
import org.citygml4j.model.gml.feature.AbstractFeature;
import org.citygml4j.model.module.citygml.CityGMLVersion;
import org.citygml4j.model.module.citygml.CoreModule;
import org.citygml4j.xml.io.CityGMLInputFactory;
import org.citygml4j.xml.io.CityGMLOutputFactory;
import org.citygml4j.xml.io.reader.CityGMLReader;
import org.citygml4j.xml.io.reader.FeatureReadMode;
import org.citygml4j.xml.io.writer.CityModelWriter;

public class ApplyXSLTWhenWriting {

	public static void main(String[] args) throws Exception {
		SimpleDateFormat df = new SimpleDateFormat("[HH:mm:ss] "); 

		System.out.println(df.format(new Date()) + "setting up citygml4j context and JAXB builder");
		CityGMLContext ctx = new CityGMLContext();
		CityGMLBuilder builder = ctx.createCityGMLBuilder();

		// we are parsing one top-level feature after the other  
		System.out.println(df.format(new Date()) + "reading CityGML file LOD2_Buildings_v100.gml chunk-wise");
		CityGMLInputFactory in = builder.createCityGMLInputFactory();
		in.setProperty(CityGMLInputFactory.FEATURE_READ_MODE, FeatureReadMode.SPLIT_PER_COLLECTION_MEMBER);

		CityGMLReader reader = in.createCityGMLReader(new File("../../datasets/LOD2_Buildings_v100.gml"));

		System.out.println(df.format(new Date()) + "creating CityGML 2.0.0 model writer");
		CityGMLOutputFactory out = builder.createCityGMLOutputFactory();
		out.setCityGMLVersion(CityGMLVersion.v2_0_0);

		// you can pass one or more XSLT stylesheets to a CityGMLOutputFactory. The stylesheets
		// are applied before writing the data. If you provide more than one stylesheet,
		// the stylesheets are automatically chained, i.e., the output of the first one is taken as input
		// for the second one, and so on. The order in which you pass the stylesheets to the CityGMLOutputFactory
		// is therefore important.
		
		// Note: If you parse the document chunk-wise (as in this example), make sure that your XSLT 
		// transformations use a local scope that matches the chunks. 
		
		// In this example, the stylesheet formats all coordinate values to have exactly three digits after the
		// the decimal separator.
		System.out.println(df.format(new Date()) + "reading XSLT stylesheet CoordinateFormatter.xsl");
		SAXTransformerFactory factory = (SAXTransformerFactory)TransformerFactory.newInstance();
		Templates stylesheet = factory.newTemplates(new StreamSource(new File("../../datasets/stylesheets/CoordinateFormatter.xsl")));
		out.setTransformationTemplates(stylesheet);

		// create a CityGML writer
		CityModelWriter writer = out.createCityModelWriter(new File("LOD2_Buildings_formatted_v200.gml"));
		writer.setPrefixes(CityGMLVersion.v2_0_0);
		writer.setDefaultNamespace(CoreModule.v2_0_0);
		writer.setSchemaLocations(CityGMLVersion.v2_0_0);
		writer.setIndentString("  ");
		writer.setHeaderComment("written by citygml4j");

		writer.writeStartDocument();

		// copy the top-level features to the target file
		while (reader.hasNext()) {
			CityGML chunk = reader.nextFeature();

			if (chunk instanceof AbstractCityObject) {
				AbstractFeature feature = (AbstractFeature)chunk;
				System.out.println(df.format(new Date()) + "applying stylesheet to feature " + feature.getId());
				
				// note that the stylesheets are automatically applied - no further action has to be taken
				writer.writeFeatureMember(feature);
			}
		}

		writer.writeEndDocument();

		reader.close();
		writer.close();

		System.out.println(df.format(new Date()) + "CityGML file LOD2_Buildings_formatted_v200.gml written");
		System.out.println(df.format(new Date()) + "sample citygml4j application successfully finished");
	}
}
